/******************************************************************************
 *
 * File:        details.h
 *
 * Description: This file contains the declaration of the Details class.
 *
 * Copyright (C) 2023 CrossControl AB
 * All rights reserved.
 * Contact: CrossControl AB (info@crosscontrol.com)
 *
 * Code Template Revision: 1.2
 *
 ******************************************************************************/

/******************************************************************************
 * Nesting Header
 ******************************************************************************/
#ifndef DETAILS_H
#define DETAILS_H

/******************************************************************************
 * Include Files
 ******************************************************************************/
#include <QFile>
#include <QList>
#include <QLocale>
#include <QMap>
#include <QObject>
#include <QRegularExpression>
#include <QStringList>
#include <QVariant>
#include <QtQml/qqmlregistration.h>

namespace WindowManager {
Q_NAMESPACE
namespace Core {

    /******************************************************************************
     * Manifest Constants, Macros, etc
     ******************************************************************************/
    const QString GROUP_DESKTOP_ENTRY              = "Desktop Entry";
    const QString GROUP_X_CCWINDOWMANAGER_SETTINGS = "X-CCWindowManager Settings";

    const QString ENTRY_NAME               = "Name";
    const QString ENTRY_ICONPATH           = "Icon";
    const QString ENTRY_TRYEXEC            = "TryExec";
    const QString ENTRY_EXEC               = "Exec";
    const QString ENTRY_X_DEFAULTREGION    = "X-DefaultRegion";
    const QString ENTRY_X_WINDOWSTATE      = "X-WindowState";
    const QString ENTRY_X_MINIMIZEDMODE    = "X-MinimizedMode";
    const QString ENTRY_X_EXECUTABLE       = "X-Executable";
    const QString ENTRY_X_SURFACECONTAINER = "X-SurfaceContainer";
    const QString ENTRY_X_QUICKBARELEMENT  = "X-QuickBarElement";

    const QString LAYOUT_DEFAULT = "DEFAULT";
    const QString REGION_DEFAULT = "QuickBar";

    /******************************************************************************
     * Class Declaration
     ******************************************************************************/
    /**
     * @brief The Details class contains data, for a specific binary, parsed from
     *  a desktop entry file.
     */
    class Details : public QObject
    {
        Q_OBJECT
        Q_PROPERTY(QString name READ name CONSTANT)
        Q_PROPERTY(QString iconPath READ iconPath CONSTANT)
        Q_PROPERTY(WindowState windowState READ windowState CONSTANT)
        Q_PROPERTY(MinimizedMode minimizedMode READ minimizedMode CONSTANT)
        Q_PROPERTY(QString surfaceContainer READ surfaceContainer CONSTANT)
        Q_PROPERTY(QString quickBarElement READ quickBarElement CONSTANT)
        Q_PROPERTY(bool executable READ executable CONSTANT)
        Q_PROPERTY(QVariantMap defaultRegions READ defaultRegionsVariantMap CONSTANT)
        QML_ELEMENT

    public:
        enum WindowState
        {
            WS_MINIMIZED,
            WS_NORMAL
        };
        Q_ENUM(WindowState)

        enum MinimizedMode
        {
            MM_TERMINATED,
            MM_RUNNING,
            MM_PAUSED
        };
        Q_ENUM(MinimizedMode)

        /**
         * @brief Default constructor of the class
         */
        Details(QObject *parent = nullptr);

        /**
         * @brief Constructor parse data from a desktop entry file
         * @param path The absolute path to the desktop entry file
         */
        Details(QString path, QObject *parent = nullptr);

        /**
         * @brief Constructor to create default details for an unknown process
         * @param binaryName The name or the binary to register
         * @param binaryPath The absolute path to the binary
         * @param execCommand The command to run the binary (can contain arguments)
         */
        Details(QString binaryName, QString binaryPath, QString execCommand, QObject *parent = nullptr);

        /**
         * @brief Returns the name of the desktop entry
         *
         * @return QString with the name
         */
        QString name(bool locale = true);

        /**
         * @brief Returns the absolute path to the icon
         * @return QString with the absolute path to the icon
         */
        QString iconPath();

        /**
         * @brief Returns the absolute path binary
         * @return QString with the absolute path binary
         */
        QString tryExec();

        /**
         * @brief Returns the command to execute the binary (can contain arguments)
         * @return QString with the command to execute the binary
         */
        QString exec();

        /**
         * @brief Returns the program of the command
         * @return QString with the program of the command
         */
        QString program();

        /**
         * @brief Returns the arguments of the command
         * @return QStringList with the arguments of the command
         */
        QStringList arguments();

        /**
         * @brief Returns default regions for a specific layout
         * @param QString Name of the layout to get default values for
         * @return QStringList with the default regions
         */
        Q_INVOKABLE QStringList defaultRegion(QString layoutName = LAYOUT_DEFAULT);

        /**
         * @brief Returns the default window state
         * @return WindowState
         */
        WindowState windowState();

        /**
         * @brief Returns what mode the client should have while minimized
         * @return MinimizedMode
         */
        MinimizedMode minimizedMode();

        /**
         * @brief Returns the flag telling if client can be executed from compositor or not
         *
         * @return bool with the value
         */
        bool executable();

        /**
         * @brief Returns absolute path to custom surface component
         * @return QString with absolute path
         */
        QString surfaceContainer();

        /**
         * @brief Returns absolute path to custom list component
         * @return QString with absolute path
         */
        QString quickBarElement();

        /**
         * @brief Returns if the parsed desktop registration is valid
         * @return bool if the parsed desktop registration is valid
         */
        bool valid();

        /**
         * @brief Returns a map of layout specific default region preferences
         * @note Reserved strings:
         * layoutId "DEFAULT": region preference for any layout not in the map
         * regionId "LAYOUT_DEFAULT" is reserved and maps to the layout's
         * own preferred default region.
         * @return QMap<QString, QStringList> where the keys are regionIds
         * and the values a list of regionIds in descending priority order
         */

        QMap<QString, QStringList> defaultRegions() const;

        /**
         * @brief Returns defaultRegions() in a QML JS understandable format
         * See defaultRegions() for more details
         * @return QVariantMap of QMap from defaultRegions()
         */
        QVariantMap defaultRegionsVariantMap() const;

    signals:

    private:
        bool parseDesktop(QString path);
        void parseLayoutRegions();
        bool validate();

        bool value(QString group, QString key, QVariant &outValue, bool locale = false);
        QString parseExec(int group);

        QMap<QString, QMap<QString, QString>>
            m_desktopEntry;
        QMap<QString, QStringList>
            m_defaultRegions;
        bool m_valid;

        static QMap<QString, WindowState>
            m_windowStates;
        static QMap<QString, MinimizedMode>
            m_minimizedModes;
    };
} // Core
} // WindowManager

#endif // DETAILS_H
