/******************************************************************************
 *
 * File:        layout.h
 *
 * Description: This file contains the declaration of Layout which is used
 * to handle information about one layout, its region(s) as well as reading and
 * writing of layouts to file in JSON format.
 *
 * JSON file handling is based on the example
 * https://doc.qt.io/qt-6.4/qtcore-serialization-savegame-example.html
 *
 * Copyright (C) 2023 CrossControl AB
 * All rights reserved.
 * Contact: CrossControl AB (info@crosscontrol.com)
 *
 * Code Template Revision: 1.2
 *
 ******************************************************************************/

/******************************************************************************
 * Nesting Header
 ******************************************************************************/
#ifndef LAYOUT_H
#define LAYOUT_H

/******************************************************************************
 * Include Files
 ******************************************************************************/
#include "region.h"
#include <QObject>
#include <QtQml/qqmlregistration.h>

namespace WindowManager {
namespace Core {

    /******************************************************************************
     * Manifest Constants, Macros
     ******************************************************************************/
    const int REGION_START_Z = 10;

    /******************************************************************************
     * Class Declaration
     ******************************************************************************/
    /**
     * @brief The Layout that handles the region information in the compositor, i.e. the
     * area where one client is displayed.
     */
    class Layout : public QObject
    {
        Q_OBJECT

        Q_PROPERTY(QString id READ id CONSTANT)
        Q_PROPERTY(QString name READ name CONSTANT)
        Q_PROPERTY(QList<Region *> regions READ regions CONSTANT)
        Q_PROPERTY(QString defaultRegion READ defaultRegion CONSTANT)
        Q_PROPERTY(QString dropHandler READ dropHandler CONSTANT)

        QML_ELEMENT
    public:
        /**
         * @brief Constructor of the class
         */
        Layout(QObject *parent = nullptr);

        /**
         * @brief Returns the id of the Layout
         * @return QString with the id
         */
        QString id();

        /**
         * @brief Sets the id of the Layout
         * @param value QString with the id
         */
        void setId(QString value);

        /**
         * @brief Returns the name of the Layout
         * @return QString with the name
         */
        QString name();

        /**
         * @brief Sets the name of the Layout
         * @param value QString with the name
         */
        void setName(QString value);

        /**
         * @brief Returns the list of regions of the Layout
         * @return QString with the name
         */
        QList<Region *> &regions();

        /**
         * @brief Returns the default region of the Layout
         * @return QString with the default region
         */
        QString defaultRegion();

        /**
         * @brief Sets the default region of the Layout
         * @param value QString with the default region
         */
        void setDefaultRegion(QString value);

        /**
         * @brief Checks if layout contains a region with given id
         * @param regionId Region to check for
         * @return True if region exists, False otherwise
         */
        Q_INVOKABLE bool regionExists(QString regionId);

        /**
         * @brief Returns the drop handler of the Layout
         * @return QString with the drop handler
         */
        QString dropHandler();

        /**
         * @brief Sets the drop handler of the Layout
         * @param value QString with the drop handler
         */
        void setDropHandler(QString value);

        /**
         * @brief Reads layouts from one or several JSON files
         * @param parent QObject based parent for ownership
         * @param dir Directory to scan for layout JSON files
         * @param layouts Resulting list of Layout objects
         */
        static void readLayouts(QObject *parent, QString dir, QList<Layout *> &layouts);

        /**
         * @brief Creates dummy Layout data and writes to one JSON file
         * @param layouts List of Layout objects to store
         */
        static void writeLayouts(QList<Layout *> layouts);

    private:
        void writeJson(QJsonObject &json);
        void readJson(QJsonObject &json);
        static void parseLayoutFile(QString filepath, QList<Layout *> &layouts, QObject *parent);

        QString m_id;
        QString m_name;
        QString m_defaultRegion;
        QString m_dropHandler;
        QList<Region *> m_regions;
    };

} // Core
} // WindowManager

#endif // LAYOUT_H
